@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -
 #
-# This file is part of gunicorn released under the MIT license. 
+# This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
 
 import os
@@ -9,8 +9,9 @@ import sys
 from gunicorn import util
 from gunicorn.app.base import Application
 
+
 class WSGIApplication(Application):
-    
+
     def init(self, parser, opts, args):
         if len(args) != 1:
             parser.error("No application module specified.")
@@ -23,10 +24,11 @@ class WSGIApplication(Application):
     def load(self):
         return util.import_app(self.app_uri)
 
+
 def run():
     """\
     The ``gunicorn`` command line runner for launcing Gunicorn with
     generic WSGI applications.
     """
     from gunicorn.app.wsgiapp import WSGIApplication
-    WSGIApplication("%prog [OPTIONS] APP_MODULE").run()
\ No newline at end of file
+    WSGIApplication("%(prog)s [OPTIONS] APP_MODULE").run()
