@@ -8,6 +8,8 @@ from __future__ import with_statement
 import os
 import sys
 from datetime import datetime
+from functools import partial
+import time
 
 # workaround on osx, disable kqueue
 if sys.platform == "darwin":
@@ -36,6 +38,7 @@ BASE_WSGI_ENV = {
     'wsgi.run_once': False
 }
 
+
 class GeventWorker(AsyncWorker):
 
     server_class = None
@@ -51,23 +54,33 @@ class GeventWorker(AsyncWorker):
         return gevent.Timeout(self.cfg.keepalive, False)
 
     def run(self):
-        self.socket.setblocking(1)
+        servers = []
+        ssl_args = {}
 
-        pool = Pool(self.worker_connections)
-        if self.server_class is not None:
-            server = self.server_class(
-                self.socket, application=self.wsgi, spawn=pool, log=self.log,
-                handler_class=self.wsgi_handler)
-        else:
-            server = StreamServer(self.socket, handle=self.handle, spawn=pool)
+        if self.cfg.is_ssl:
+            ssl_args = dict(server_side=True,
+                    do_handshake_on_connect=False, **self.cfg.ssl_options)
+
+        for s in self.sockets:
+            s.setblocking(1)
+            pool = Pool(self.worker_connections)
+            if self.server_class is not None:
+                server = self.server_class(
+                    s, application=self.wsgi, spawn=pool, log=self.log,
+                    handler_class=self.wsgi_handler, **ssl_args)
+            else:
+                hfun = partial(self.handle, s)
+                server = StreamServer(s, handle=hfun, spawn=pool, **ssl_args)
+
+            server.start()
+            servers.append(server)
 
-        server.start()
         pid = os.getpid()
         try:
             while self.alive:
                 self.notify()
 
-                if  pid == os.getpid() and self.ppid != os.getppid():
+                if pid == os.getpid() and self.ppid != os.getppid():
                     self.log.info("Parent changed, shutting down: %s", self)
                     break
 
@@ -77,9 +90,27 @@ class GeventWorker(AsyncWorker):
             pass
 
         try:
-            # Try to stop connections until timeout
-            self.notify()
-            server.stop(timeout=self.cfg.graceful_timeout)
+            # Stop accepting requests
+            [server.stop_accepting() for server in servers]
+
+            # Handle current requests until graceful_timeout
+            ts = time.time()
+            while time.time() - ts <= self.cfg.graceful_timeout:
+                accepting = 0
+                for server in servers:
+                    if server.pool.free_count() != server.pool.size:
+                        accepting += 1
+
+                # if no server is accepting a connection, we can exit
+                if not accepting:
+                    return
+
+                self.notify()
+                gevent.sleep(1.0)
+
+            # Force kill all active the handlers
+            self.log.warning("Worker graceful timeout (pid:%s)" % self.pid)
+            server.stop(timeout=1)
         except:
             pass
 
@@ -106,12 +137,12 @@ class GeventResponse(object):
     headers = None
     response_length = None
 
-
     def __init__(self, status, headers, clength):
         self.status = status
         self.headers = headers
         self.response_length = clength
 
+
 class PyWSGIHandler(pywsgi.WSGIHandler):
 
     def log_request(self):
@@ -129,9 +160,11 @@ class PyWSGIHandler(pywsgi.WSGIHandler):
         env['RAW_URI'] = self.path
         return env
 
+
 class PyWSGIServer(pywsgi.WSGIServer):
     base_env = BASE_WSGI_ENV
 
+
 class GeventPyWSGIWorker(GeventWorker):
     "The Gevent StreamServer based workers."
     server_class = PyWSGIServer
